/*
btupdt.c Version 0.1.0  Btree update record
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* return code 0 is good update */
/* return code 1 is record not found */

#include <bthash.h>

int btupdt(btreefmt *bthdr, long ofst,
   unsigned char *rcd)
   {
   int sigrslt;
   int btgrslt;
   unsigned char *p,*q,*r;
   unsigned char *buf;
   bthdr->reason = 0;
   sigrslt = strcmp(bthdr->signature,BT_SIGNATURE);
   if (sigrslt)
      {
      fprintf(stderr,"btupdt: non-btree "
         "update\n");
      bthdr->reason = BTREE_ERR + 2206;
      return(BTREE_ERR + 2206);
      } /* not btree database */
   p = (unsigned char *) rcd + bthdr->dletofst;
   if (*p == DELETED)
      {
      fprintf(stderr,"btupdt: updating "
	 "record has delete byte set\n");
      fprintf(stderr,"btupdt: call "
	 "btdlet for delete\n");
      bthdr->reason = BTREE_ERR + 2205;
      return(BTREE_ERR + 2205);
      } /* delete byte set */
   if (ofst == ROOT
      && bthdr->head->next != bthdr->tail)
      {
      fprintf(stderr,"btupdt: inserting at root "
	 "with non-empty stack\n");
      return(BTREE_ERR + 2201);
      } /* insert loop */
   buf = (unsigned char *) malloc(bthdr->rcdlen + 16);
   if (buf == NULL)
      {
      fprintf(stderr,"btupdt: out of memory\n");
      return(BTREE_ERR + 2202);
      } /* out of mem */
   p = (unsigned char *) rcd + bthdr->keyofst;
   btgrslt = btget(bthdr,ofst,p,buf);
   free(buf);
   if (btgrslt == FRSTPAPA) return(UPDTERR);
   else if (btgrslt) return(btgrslt);
   p = (unsigned char *) bthdr->blk + bthdr->blkhdr
      + (bthdr->rcdnum * bthdr->rcdnode)
      - bthdr->rcdnode + bthdr->rcdhdr;
   q = (unsigned char *) p + bthdr->rcdlen;
   r = (unsigned char *) rcd;
   if (*(p + bthdr->dletofst) == DELETED)
      {
      fprintf(stderr,"btupdt: matching "
	 "record has delete byte set\n");
      bthdr->reason = BTREE_ERR + 2204;
      return(BTREE_ERR + 2204);
      } /* deleting twice */
   while (p < q) *p++ = *r++;
   btputblk(bthdr,bthdr->ofst,bthdr->blk);
   if (bthdr->reason) return(BTREE_ERR + 2203);
   p = (unsigned char *) bthdr->blk;
   q = (unsigned char *) bthdr->blk + bthdr->blksz;
   while (p < q) *p++ = '\0';
   bthdr->ofst = (long) 0L;
   return(UPDTGOOD);
   } /* btupdt */
